﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;

namespace Analog_Input_Scan
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            string IP = "";
            bool ret = false;
            bool quitIP = false;
            bool quit = false;

            IntPtr hHS;

            UInt16 gVal = 0;
            Int16 oVal = 0;

            UInt16[] gVarr = new UInt16[8];
            Int16[] oVarr = new Int16[8];
            short ChCnt = 8;

            short Gain = 0;
            uint ulleng = 0;
            ushort BufferStatus = 0;
            UInt32 readsize = 0;



            short TrigMode = 0; 
            UInt32 targetCnt = 0;
            int sampleRate = 30000;

            //unsigned long totalRead=0,totalReceived=0;
            short DataTransMethod = 0;
            short AutoRun = 0;

            while (quitIP == false)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quitIP = true;
                    Thread.Sleep(100);
                }
            }


            hHS = HSDAQNet.Sys.HS_Device_Create(IP);

            if (hHS != IntPtr.Zero)
            {
                Console.WriteLine("HSDAQ SDK Ver= " + HSDAQNet.Sys.GetSDKVersion().ToString());
                Console.WriteLine("ET7H16 Firmware Ver= " + HSDAQNet.Sys.GetFirmwareVersion(hHS).ToString());

                HSDAQNet.HSIO.HS_SetAIScanParam(hHS, ChCnt, Gain, TrigMode, sampleRate, targetCnt, DataTransMethod, AutoRun);

                HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref ChCnt, ref Gain, ref TrigMode, ref sampleRate, ref targetCnt, ref DataTransMethod, ref AutoRun);
                Console.WriteLine("Ch=" + ChCnt + ",Gain=" + Gain + ",TrigMode=" + TrigMode + ",sampleRate=" + sampleRate + ",targetCnt=" + targetCnt);

                for (int ch = 0; ch < 8; ch++)
                {
                    HSDAQNet.Config.HS_ReadGainOffset(hHS, ch, Gain, ref gVal, ref oVal);

                    gVarr[ch] = gVal;
                    oVarr[ch] = oVal;
                    Console.WriteLine("[ch" + ch + "]Gain = " + gVal + " , Offset = " + oVal);
                }


                if (!ret)
                {
                    Console.WriteLine("Error code1 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                }
                else
                {
                    Console.WriteLine(System.DateTime.Now);
                    ret = HSDAQNet.HSIO.HS_StartAIScan(hHS);
                    if (ret == false)
                    {
                        Console.WriteLine("Error code2 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                    }
                    Console.WriteLine("Press 'Q' to stop capturing data");

                    while (quit == false)
                    {
                        //Console.Clear();
                        //Console.Write("capturing data...");
                        ret = HSDAQNet.HSIO.HS_GetAIBufferStatus(hHS, ref BufferStatus, ref ulleng);
                        //Console.Write(readsize + "\r");

                        if (ret == false)
                        {
                            Console.WriteLine("Error code3 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                        }
                        else
                        {
                            if (BufferStatus > 2 && (BufferStatus >> 7 & 0x01) != 1) //AI buffer overflow
                            {

                                //  2:  AD_BUF_OVERFLOW	
                                //  4:  AD_SCAN_STOP
                                //  8:  AD_DATA_SAMPLING_TIMEOUT

                                Console.WriteLine("BufferStatus Error<" + BufferStatus + ">," + ulleng);
                                break;
                            }

                            uint[] dataBuffer = new uint[ulleng * ChCnt];
                            readsize += HSDAQNet.HSIO.HS_GetAIBufferHex(hHS, dataBuffer, ulleng);

                        }


                        if (ulleng > 0)
                        {
                            if (Console.KeyAvailable)
                            {
                                if (Console.ReadKey(true).Key == ConsoleKey.Q)
                                {
                                    break;
                                }
                            }
                        }

                    }
                    
                    HSDAQNet.HSIO.HS_StopAIScan(hHS);
                    HSDAQNet.Sys.HS_Device_Release(hHS);
                    Console.WriteLine("\r\nAP total read = " + readsize + "\r\n");
                }

                Console.WriteLine("Press 'Q' to quit program");

                while (Console.ReadKey(true).Key != ConsoleKey.Q) ;
            }            
        }
    }
}
