﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;

namespace Analog_Input_float_Ext_CLK
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            string IP = "";
            bool ret = false;
            bool quitIP = false;
            bool quit = false;
            bool triggerEvent = false;
            IntPtr hHS;
            IntPtr hcom;
            UInt16 gVal = 0;
            Int16 oVal = 0;

            UInt16[] gVarr = new UInt16[8];
            Int16[] oVarr = new Int16[8];
            short ChCnt = 8;

            string I7Kcom = "COM13,9600,N,8,1";


            short Gain = 0;
            uint ulleng = 0;
            ushort BufferStatus = 0;
            UInt32 readsize = 0;
            UInt32 totalRecv = 0;
            short extCLKCnt = 20000;


            short TrigMode = 1; //external clock trigger
            UInt32 targetCnt = 20000;
            int sampleRate = 30000;

            //unsigned long totalRead=0,totalReceived=0;
            short DataTransMethod = 0;
            short AutoRun = 0;
            Console.WriteLine("This demo uses I-7088 as a trigger module.\r\nUsers can change the trigger mode by themselves, such as using a signal generator, just remove the part about controlling I-7088 below.");
            while (quitIP == false)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quitIP = true;
                    Thread.Sleep(100);
                }
            }


            hHS = HSDAQNet.Sys.HS_Device_Create(IP);
            if (hHS != IntPtr.Zero)
            {
                hcom = PACNET.UART.Open(I7Kcom);//For I-7088

                Console.WriteLine("HSDAQ SDK Ver= " + HSDAQNet.Sys.GetSDKVersion().ToString());
                Console.WriteLine("ET7H16 Firmware Ver= " + HSDAQNet.Sys.GetFirmwareVersion(hHS).ToString());

                Directory.CreateDirectory("C:\\test");
                FileStream fileStream = new FileStream("C:\\test\\test.txt", FileMode.OpenOrCreate);

                fileStream.Close();

                ret = HSDAQNet.HSIO.HS_SetAIScanParam(hHS, ChCnt, Gain, TrigMode, sampleRate, targetCnt, DataTransMethod, AutoRun);

                if (!ret)
                {
                    Console.WriteLine("Error code1 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                }
                else
                {
                    HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref ChCnt, ref Gain, ref TrigMode, ref sampleRate, ref targetCnt, ref DataTransMethod, ref AutoRun);
                    Console.WriteLine("Ch=" + ChCnt + ",Gain=" + Gain + ",TrigMode=" + TrigMode + ",sampleRate=" + sampleRate + ",targetCnt=" + targetCnt);
                }
                

                for (int ch = 0; ch < 8; ch++)
                {
                    HSDAQNet.Config.HS_ReadGainOffset(hHS, ch, Gain, ref gVal, ref oVal);

                    gVarr[ch] = gVal;
                    oVarr[ch] = oVal;
                }


                if (!ret)
                {
                    Console.WriteLine("Error code1 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                }
                else
                {


                    ret = HSDAQNet.HSIO.HS_StartAIScan(hHS);
                    if (ret == false)
                    {
                        Console.WriteLine("Error code2 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                    }
                    Console.WriteLine("Press any key to trigger an external signal....");

                    {
                        while (quit == false)
                        {
                            ret = HSDAQNet.HSIO.HS_GetAIBufferStatus(hHS, ref BufferStatus, ref ulleng);
                            float[] FdataBuffer = new float[ulleng * 8 * 2];

                            if (ret == false)
                            {
                                Console.WriteLine("Error code3 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                            }
                            else
                            {
                                if (BufferStatus > 2 && (BufferStatus >> 7 & 0x01) != 1) //AI buffer overflow
                                {

                                    //  2:  AD_BUF_OVERFLOW	
                                    //  4:  AD_SCAN_STOP
                                    //  8:  AD_DATA_SAMPLING_TIMEOUT

                                    Console.WriteLine("BufferStatus Error<" + BufferStatus + ">," + ulleng);
                                    break;
                                }

                                //size = targetCnt;


                                if (ulleng == targetCnt)
                                {
                                    readsize = HSDAQNet.HSIO.HS_GetAIBuffer(hHS, FdataBuffer, ulleng);

                                    quit = true;

                                }

                            }
                            /******************This part controls I-7088*********************/
                            if (Console.KeyAvailable)
                            {


                                Thread.Sleep(100);
                                if (hcom == (IntPtr)(-1))
                                {
                                    uint ec = PACNET.ErrHandling.GetLastError();
                                    Console.WriteLine(((PACNET.ErrCode)ec).ToString() + "I-7088 COM Port open error\r\nError Code: 0x" + ec.ToString("X"));
                                }

                                ret = PACNET.PWM.SetPWMFrequency(hcom, PACNET.PAC_IO.PAC_REMOTE_IO(1), 0, 10000);
                                if (!ret)
                                {
                                    uint ec = PACNET.ErrHandling.GetLastError();
                                    Console.WriteLine(((PACNET.ErrCode)ec).ToString() + "\nError Code: 0x" + ec.ToString("X"));
                                }
                                else
                                {
                                    PACNET.PWM.SetPWMPulseCount(hcom, PACNET.PAC_IO.PAC_REMOTE_IO(1), 0, extCLKCnt);
                                    PACNET.PWM.SetPWMStart(hcom, PACNET.PAC_IO.PAC_REMOTE_IO(1), 1);    //Start PWM
                                    Console.WriteLine("The external CLK signal is triggered");
                                }
                                Thread.Sleep(1000);
                                PACNET.PWM.SetPWMStart(hcom, PACNET.PAC_IO.PAC_REMOTE_IO(1), 0);    //OFF PWM
                                //triggerEvent = true;
                            }
                            /*****************************************************************/
                            if (triggerEvent)
                            {

                                Console.WriteLine("Press 'Q' to quit program");
                                while (triggerEvent == true)
                                {
                                    //    Console.ReadLine();
                                    //if (Console.ReadLine() == "Q" || Console.ReadLine() == "q")
                                    if (Console.ReadKey().Key == ConsoleKey.Q)
                                    {
                                        triggerEvent = false;
                                        quit = true;
                                        break;
                                    }

                                    else
                                        Thread.Sleep(10);
                                }
                            }

                            Thread.Sleep(200);

                        }
                        HSDAQNet.HSIO.HS_StopAIScan(hHS);
                        HSDAQNet.Sys.HS_Device_Release(hHS);
                        PACNET.UART.Close(hcom);//For I-7088
                        Console.WriteLine("\r\nAP total read = " + readsize + "\r\n");
                        Console.WriteLine("Press 'Q' to quit program");
                        while (Console.ReadKey(true).Key != ConsoleKey.Q) ;
                    }


                }
            }
         }
    }
}
