﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;
using HSDAQNet;

namespace Analog_Sync_Input_Scan
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            string IP = "";
            bool ret = false;
            bool quitIP = false;
            bool quit = false;

            IntPtr hHS;
            short ChCnt = 8;

            UInt32 totalRecv = 0;
            short Gain = 0;
            uint ulFramelength = 0;
            ushort BufferStatus = 0;
            UInt32 readsize = 0;

            short TrigMode = 0; 
            UInt32 targetCnt = 80000;
            int sampleRate = 1000;

            //unsigned long totalRead=0,totalReceived=0;
            short DataTransMethod = 0;
            short AutoRun = 0;

            while (quitIP == false)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quitIP = true;
                    Thread.Sleep(100);
                }
            }


            hHS = Sys.HS_Device_Create(IP);

            Console.WriteLine("HSDAQ SDK Ver= " + Sys.GetSDKVersion().ToString());
            Console.WriteLine("ET7H16 Firmware Ver= " + Sys.GetFirmwareVersion(hHS).ToString());

            ret = HSIO.HS_SetAIScanParam(hHS, ChCnt, Gain, TrigMode, sampleRate, targetCnt, DataTransMethod, AutoRun);
            if (!ret)
            {
                Console.WriteLine("HS_SetAIScanParam Error code1 0x" + ErrHandling.GetLastError().ToString("x8"));
            }
            HSIO.HS_GetAIScanParam(hHS,ref ChCnt,ref Gain,ref TrigMode,ref sampleRate,ref targetCnt,ref DataTransMethod,ref AutoRun);
            Console.WriteLine("Ch=" + ChCnt + ",Gain=" + Gain + ",TrigMode=" + TrigMode + ",sampleRate=" + sampleRate + ",targetCnt=" + targetCnt);

            //Step 4: Set counter configuration
            UInt32 dwCNTmode = (UInt32)IO.CNTCong.CNT_DISABLE;
            UInt32 dwCNTInitValue = 0;
            ret=IO.HS_SetCounterConfig(hHS,0, dwCNTmode, dwCNTInitValue, 0);
            if (!ret)
            {
                Console.WriteLine("HS_SetCounterConfig Error code1 0x" + ErrHandling.GetLastError().ToString("x8"));
            }

            Thread.Sleep(10);
            Console.WriteLine("HS_SetCounterConfig");
            //Step 5: Read counter configuration
            UInt32 uiReserved = 0;
            IO.HS_GetCounterConfig(hHS, 0, ref dwCNTmode, ref dwCNTInitValue, ref uiReserved);
            Console.WriteLine("Counter parameters: Channel:0, mode" + dwCNTmode.ToString()+" Init value "+dwCNTInitValue.ToString());		

            {

            UInt16 SyInTypeNum = 2;
            UInt32 SyncInheader = 1;//index
            UInt16 AcutArryAcount = 0;
            UInt32 options = (UInt32)HSIO.SYNC_STATUS.SYNC_ENABE;

            UInt16[] SyInChTypeArray = new UInt16[]{(UInt16)HSIO.SYNC_IN_TYPE.SYNC_IN_AI_HEX ,(UInt16)HSIO.SYNC_IN_TYPE.SYNC_IN_DWORD_CNT};
            UInt16[] SyInChNumArray = new UInt16[] { (UInt16)ChCnt, 1 };

		    //Step 6: HS_SetSyncInScanParam sets the synchronous configuration (header,input type and input number for synchronous ethernet frame)
            ret = HSIO.HS_SetSyncInScanParam(hHS,SyncInheader,SyInChNumArray, SyInChTypeArray,(UInt16) SyInTypeNum,(UInt32)options, 0);
		    
		    if(ret==false)
		    {
                Console.WriteLine("HS_SetSyncInScanParam Error code 0x" + ErrHandling.GetLastError().ToString("x8"));
		    }

            HSIO.HS_GetSyncInScanParam(hHS, ref SyncInheader, SyInChNumArray, SyInChTypeArray, SyInTypeNum, ref AcutArryAcount, ref options, ref uiReserved);
            Console.WriteLine("Synchronous parameters header type:"+SyncInheader.ToString()+" AcutalArryAcount:"+ AcutArryAcount.ToString()+" options:"+ options.ToString());	

		    int i;
		    for(i=0;i<AcutArryAcount;i++)
                Console.WriteLine(i.ToString()+" Input channel number:"+SyInChNumArray[i].ToString()+" type:"+ SyInChTypeArray[i].ToString());
            Thread.Sleep(10);
                
              ret = HSIO.HS_StartAIScan(hHS);
                if (ret == false)
                {
                    Console.WriteLine("Error code2 0x" + ErrHandling.GetLastError().ToString("x8"));
                }
                Console.WriteLine("Start capturing data by pressing any key other than 'Q', then press 'Q' to stop capturing data");

                while (quit == false)
                {
                    //Console.Clear();
                    //Console.Write("capturing data...");
                    ret = HSIO.HS_GetSyncInBufferStatus(hHS, ref BufferStatus, ref ulFramelength);

                    if (ret == false)
                    {
                        Console.WriteLine("Error code3 0x" + ErrHandling.GetLastError().ToString("x8"));
                    }
                    else
                    {
                        //Console.WriteLine("BufferStatus Error<" + BufferStatus + ">," + ulFramelength);
                        if (BufferStatus >=2) //AI buffer overflow
                        {

                            //  2:  AD_BUF_OVERFLOW	
                            //  4:  AD_SCAN_STOP
                            //  8:  AD_DATA_SAMPLING_TIMEOUT

                            Console.WriteLine("BufferStatus Error<" + BufferStatus + ">," + ulFramelength);
                            break;
                        }

                        if (ulFramelength > 0)
                        {
                            UInt32 temp = 0;
                            
                            //DAQ_TIME *dtheaderbuf=NULL;

                            UInt32[] dwheaderbuf = null;
                            UInt32[] wAIbuffer = null; //new UInt32[ulFramelength];
                            UInt32[] wDICNTbuffer = null; //new UInt32[ulFramelength];
                            UInt32[] wCNTbuffer = null; //new UInt32[ulFramelength];

                            switch (SyncInheader) //dcalare header array
                            {
                                case 0:
                                default:
                                    break;
                                case 1:
                                    dwheaderbuf = new UInt32[ulFramelength];
                                    break;
                            }

                            for (i = 0; i < SyInTypeNum; i++)
                            {
                                switch (SyInChTypeArray[i])
                                {
                                    case 0: //HSIO.SYNC_IN_TYPE.SYNC_IN_AI:  //2bytes (Hex to float)	
                                    case 1: //HSIO.SYNC_IN_TYPE.SYNC_IN_AI_HEX: //2bytes
                                        temp = ulFramelength * SyInChNumArray[i];
                                        wAIbuffer = new UInt32[temp];
                                        //memset(wAIbuffer,0x0,sizeof(DWORD)*temp);
                                        break;
                                    case 4: //HSIO.SYNC_IN_TYPE.SYNC_IN_DWORD_DI_CNT: //4bytes
                                        temp = ulFramelength * SyInChNumArray[i];
                                        wDICNTbuffer = new UInt32[temp];
                                        //memset(wDICNTbuffer,0x0,sizeof(DWORD)*temp);
                                        break;
                                    case 5: // HSIO.SYNC_IN_TYPE.SYNC_IN_DWORD_CNT:  //4bytes                                     
                                        temp = ulFramelength * SyInChNumArray[i];
                                        wCNTbuffer = new UInt32[temp];
                                        //memset(wCNTbuffer,0x0,sizeof(DWORD)*temp);
                                        break;
                                    default:
                                        break;
                                }
                            }                           
                            readsize = HSIO.HS_GetSyncInBuffer1D(hHS, dwheaderbuf, wAIbuffer, null, null, wDICNTbuffer, wCNTbuffer, null, null, ulFramelength);
                            //Console.WriteLine("<" + readsize + ">," + readsize);
                            if(readsize >0)
						    {									
							    //int i=0;
							    int j=0;
							    int k=0;
							    UInt32 dwtemp=0;
							    UInt32 dwCNTtemp=0;
                                UInt32 CH1checkerror = 0;
                                UInt32 otherCHcheckerror = 0;
	
							    for(k=0;k<ulFramelength;k++) //read all frame data
							    {
								    switch(SyncInheader) //read header 
								    {
									    case 0:
   									    default:							
    										break;
	    								case 1:					
										if(dwtemp==0)
											dwtemp=dwheaderbuf[k];
										else
										{
											if(dwheaderbuf[k]-dwtemp>1)
											{
												otherCHcheckerror++;
                                                Console.WriteLine("header index error " + otherCHcheckerror.ToString() + " [" +dwtemp.ToString()+","+ dwheaderbuf[k].ToString()+"]");
												//quit=true;	
												//goto _quit;
											}
											dwtemp=dwheaderbuf[k];
										}
										break;									
								    }
								    //printf("[%lu]H:%lu\r\n",totalRecv+k, dwheaderbuf[k]);
								    int arrayind=0;
								    for(i=0;i<SyInTypeNum;i++) //read the array data of each input type on a frame data
								    {
									    switch(SyInChTypeArray[i]) //read input data 
									    {
										    case 0: //SYNC_IN_AI:  //2bytes (Hex to float)	
										    case 1: //SYNC_IN_AI_HEX: //2bytes												            
											    /*for(j=0;j<SyInChNumArray[i];j++)
											    {
												    arrayind=k*SyInChNumArray[i]+1;
												    if(wAIbuffer[arrayind]>0x1C00 || wAIbuffer[arrayind]<0x1a00)
												    {
													    CH1checkerror++;
                                                        Console.WriteLine("CH1 error=" + CH1checkerror.ToString() + ",AI=" + wAIbuffer[arrayind].ToString());													    
													    //printf("CH0:0x%x CH2:0x%x CH3:0x%x CH4:0x%x CH5:0x%x\r\n",wAIbuffer[k*SyInChNumArray[i]],wAIbuffer[arrayind+1],wAIbuffer[arrayind+2],wAIbuffer[arrayind+3],wAIbuffer[arrayind+4]);													    
                                                        Console.WriteLine("Framelength=" + ulFramelength.ToString() + ",header=" + dwheaderbuf[k].ToString());
													    //quit=true;	
													    //goto _quit;
												    }
											    }*/
											    break;
                                            case 4: //SYNC_IN_DWORD_DI_CNT: //4bytes
											/*
											for(j=0;j<SyInChNumArray[i];j++)
												printf("%0lu ",wDICNTbuffer[k][j]);*/
											    break;
										    case 5: //SYNC_IN_DWORD_CNT:  //4bytes
											/*
											    for(j=0;j<SyInChNumArray[i];j++)
												    printf("%0lu ",wCNTbuffer[k][j]);*/
											    arrayind=k*SyInChNumArray[i];
											    if(dwCNTtemp==0)
												    dwCNTtemp=wCNTbuffer[arrayind];
											    else
											    {												    
                                                    Console.WriteLine("CNT:" + wCNTbuffer[arrayind].ToString("x8"));
												    /*if(wCNTbuffer[arrayind]-dwCNTtemp!=1 && wCNTbuffer[arrayind]-dwCNTtemp!=0)
												    {
													    CNTcheckerror++;
													    printf("H:%lu CNT error %lu, [%lu, %lu]\r\n",dwheaderbuf[k],CNTcheckerror,dwCNTtemp,wCNTbuffer[arrayind]);
													    //quit=true;	
													    //goto _quit;
												    }*/
												    dwCNTtemp=wCNTbuffer[arrayind];
											    }
                                                //Console.WriteLine(wCNTbuffer[arrayind].ToString("x8"));
											    //printf("%lu ", wCNTbuffer[k][0]);
											    break;
										    default:
											    break;
									    }

								    }

							    }
    
						    }
						    else
                                Console.WriteLine("Error code2 0x" + ErrHandling.GetLastError().ToString("x8"));									        
                        }

                    }

                    totalRecv = totalRecv + readsize;
                    
                    if (targetCnt > 0 && totalRecv >= targetCnt) //targetCnt >0 : N sample mode, 0: continue mode
                    {
                        break;
                    }
                    if( Console.KeyAvailable ) // since .NET 2.0  
                    {
                        if (Console.ReadKey(true).Key == ConsoleKey.Q)
                        {                            
                            break;
                        }    
                    }
                    
                }

                HSIO.HS_StopAIScan(hHS);
                //HSIO.HS_ClearSyncInBuffer(hHS);
                Sys.HS_Device_Release(hHS);
                Console.WriteLine("\r\nAP total read = " + totalRecv+"\r\n");
            }

            Console.WriteLine("Press 'Q' to quit program");

            while (Console.ReadKey(true).Key != ConsoleKey.Q) ;
            
        }
    }
}
