﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;
using System.Windows.Forms.DataVisualization.Charting;


namespace HSDAQ_FFT_Demo
{
    public partial class Form1 : Form
    {
        public IntPtr hHS = IntPtr.Zero;
        Series[] chSeries = new Series[8];
        Color[] colorArr = { Color.Blue, Color.Yellow, Color.Green, Color.Red, Color.Black, Color.Aqua, Color.Wheat, Color.Pink };
        bool is_sampling = false;
        short MAX_ch = 1;
        short chCnt = 1, Gain = 1, Mode = 0, DataTrnsMethod = 0, AitoRun = 0;
        int SampleRate = 30000;
        uint targetCnt = 0;
        uint DATA_SIZE = 16384;
        float[] f_Data = new float[1460 * 1000];


        public Form1()
        {
            InitializeComponent();
        }

        private void btn_Connect_Click(object sender, EventArgs e)
        {
            if (btn_Connect.Text == "Connect")
            {
                hHS = HSDAQNet.Sys.HS_Device_Create(txt_IP.Text);
                btn_Connect.Text = "Disconnect";
                btn_Start.Enabled = true;

                //Setting Module
                HSDAQNet.HSIO.HS_SetAIScanParam(hHS, chCnt, Gain, Mode, SampleRate, targetCnt, DataTrnsMethod, AitoRun);
                Thread.Sleep(100);
                HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref chCnt, ref Gain, ref Mode, ref SampleRate, ref targetCnt, ref DataTrnsMethod, ref AitoRun);
                list_Setting.Items.Add("Channels : " + chCnt);
                list_Setting.Items.Add("Gain : " + Gain);
                list_Setting.Items.Add("Mode : Continue mode");
                list_Setting.Items.Add("SampleRate : " + SampleRate + " Hz");
                list_Setting.Items.Add("targetCnt : " + (int)targetCnt);

                /*--------------------------------Init chart--------------------------------------*/
                chart_FFT.Series.Clear();                               //clear chart_FFT
                for (int i = 0; i < MAX_ch; i++)
                {
                    string seriesName = "Channel" + i;
                    chSeries[i] = new Series(seriesName);               // Create a new series to hold the data.
                    chSeries[i].ChartType = SeriesChartType.Line;       // Using a Line Chart
                    chSeries[i].Color = colorArr[i];                    // Setting the Line Color
                    chSeries[i].BorderWidth = 2;                        // Setting Line Width
                    chart_FFT.Series.Add(chSeries[i]);
                }
                chart_FFT.ChartAreas[0].AxisX.Title = "Hz";
                chart_FFT.ChartAreas[0].AxisY.Title = "Amplitude (units)";
                chart_FFT.ChartAreas[0].AxisX.Minimum = 0;              // Set the minimum value of X-axis
                chart_FFT.ChartAreas[0].AxisX.Maximum = SampleRate / 2; // Setting the maximum value of X-axis
                chart_FFT.ChartAreas[0].AxisY.Minimum = -10;            // Set the minimum value of Y-axis
                chart_FFT.ChartAreas[0].AxisY.Maximum = 10;             // Setting the maximum value of Y-axis
                chart_FFT.ChartAreas[0].CursorX.IsUserSelectionEnabled = true;
                chart_FFT.ChartAreas[0].CursorY.IsUserSelectionEnabled = true;
                chart_FFT.ChartAreas[0].AxisX.ScaleView.Zoomable = true;
                chart_FFT.ChartAreas[0].AxisY.ScaleView.Zoomable = true;
                chart_FFT.Invalidate();                                 // Compulsory chart redrawing
                chart_FFT.Show();                                       // Present the chart on the window
                /*--------------------------------------------------------------------------------*/
            }
            else if (btn_Connect.Text == "Disconnect")
            {
                HSDAQNet.Sys.HS_Device_Release(hHS);
                btn_Connect.Text = "Connect";
                btn_Start.Enabled = false;
            }
        }

        private void btn_Start_Click(object sender, EventArgs e)
        {
            if (btn_Start.Text == "Start")
            {
                btn_Start.Text = "Stop";
                btn_Connect.Enabled = false;
                HSDAQNet.HSIO.HS_StartAIScan(hHS);
                is_sampling = true;

                Run_Monitor_FFT();
            }
            else if (btn_Start.Text == "Stop")
            {
                btn_Start.Text = "Start";
                btn_Connect.Enabled = true;
                is_sampling = false;
                HSDAQNet.HSIO.HS_StopAIScan(hHS);
            }
        }
        
        public void Run_Monitor_FFT()
        {
            bool ret = false;
            double[] f = new double[16384];
            double[] rf = new double[16384];
            double nyquistFrequency = SampleRate / 2.0;
            UInt16 Buffer_status = 0;
            UInt32 Buffer_Cnt = 0;
            UInt32 recv_Cnt = 0;

            while (is_sampling)
            {
                ret = HSDAQNet.HSIO.HS_GetAIBufferStatus(hHS, ref Buffer_status, ref Buffer_Cnt);
                if (ret == false)
                {
                    HSDAQNet.HSIO.HS_StopAIScan(hHS);
                    MessageBox.Show("HS_GetAIBufferStatus failed!!! \r\nError Code :  0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                    return;
                }
                else
                {
                    if ((Buffer_status & 0x20) == 0x20)
                    {
                        MessageBox.Show("Time Out!!! Error Code : 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                        HSDAQNet.HSIO.HS_StopAIScan(hHS);
                        break;
                    }
                    if (is_sampling == false)
                    {
                        break;
                    }
                    if (Buffer_Cnt > DATA_SIZE)
                    {
                        recv_Cnt = HSDAQNet.HSIO.HS_GetAIBuffer(hHS, f_Data, DATA_SIZE);
                        
                        for (int i = 0; i < DATA_SIZE; i++)
                        {
                            f[i] = f_Data[i];//float array to double array
                        }

                        rf = FftSharp.Transform.FFTmagnitude(f);    // FFT

                        chSeries[0].Points.Clear();
                        int fftSize = rf.Length;
                        double[] frequencyAxis = new double[fftSize];
                        
                        for (int i = 0; i < fftSize; i++)
                        {
                            frequencyAxis[i] = i * nyquistFrequency / (fftSize - 1);
                            chSeries[0].Points.AddXY(frequencyAxis[i], rf[i]);
                        }
                    }
                    Application.DoEvents();
                }
            }
        }
    }
}
