﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;

namespace PETAR400_AI_Log_Continue
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            bool quit = false, ret = false;
            string IP = "";
            IntPtr hHS = IntPtr.Zero;

            Console.WriteLine("!!This program demonstrates how to use continue mode for save file!!");
            Console.WriteLine("PET-AR400 has 4 AI channel");
            while (!quit)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quit = true;
                    Thread.Sleep(100);
                }
            }
            quit = false;
            hHS = HSDAQNet.Sys.HS_Device_Create(IP);
            if (hHS == null || hHS == IntPtr.Zero)
            {
                Console.WriteLine("Connect Fail");
                Console.WriteLine("\nPress any key to close...");
                Console.ReadKey();
                return;
            }
            string ModuleName = HSDAQNet.Sys.HS_GetModelName(hHS);
            Console.WriteLine("Module Name      : " + ModuleName);
            Console.WriteLine("Firmware Version : " + HSDAQNet.Sys.GetFirmwareVersion(hHS));
            Console.WriteLine("HSDAQ Version    : V." + HSDAQNet.Sys.GetSDKVersion());
            Console.WriteLine("HSDAQNet Version : V." + HSDAQNet.Sys.GetHSDAQNetVersion());
            /* For details, please refer to the API manual
             * AI CH input Range : 1 ~ 4
             * Gain Range        : 0
             * Mode Range        : 0
             * Sample rate       : 
             *  ch=1    : 12.8k/16k/32k/64k/128k Hz
             *  ch=2    : 12.8k/16k/32k/64k Hz
             *  ch=3/4  : 12.8k/16k/32k Hz
             * targetcnt         : 0 (continue mode)
             * filetype          : 0(bin)、1(txt)、3(TDMS float)、4(TDMS Hex)
             * fileinterval      : min
            */
            short ch = 4, gain = 0, mode = 0, DTM = 0, Autorun = 0;
            int samplerate = 32000;
            uint targetcnt = 0;
            string path = Directory.GetCurrentDirectory();
            int fileinterval = 2, filetype = 1;


            HSDAQNet.HSIO.HS_SetAIScanParam(hHS, ch, gain, mode, samplerate, targetcnt, DTM, Autorun);
            Thread.Sleep(10);
            HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref ch, ref gain, ref mode, ref samplerate, ref targetcnt, ref DTM, ref Autorun);
            Console.WriteLine("Channel    : " + ch);
            Console.WriteLine("Gain       : " + gain);
            Console.WriteLine("Mode       : " + mode);
            Console.WriteLine("Samplerate : " + samplerate);
            Console.WriteLine("targetcnt  : " + targetcnt);
            Console.WriteLine("---------------------------------------------------------");
            Console.WriteLine("Start!");
            Console.WriteLine("Press 'Q' to stop!");
            HSDAQNet.DATALOG.HS_StartLogger(hHS, path, fileinterval, filetype);
            while (!quit)
            {
                uint BufStatus = 0;
                uint BufferCnt = 0;
                bool result = HSDAQNet.HSIO.HS_GetTotalSamplingStatus(hHS, ref BufferCnt, ref BufStatus);
                if (result)
                {
                    if ((BufStatus & 0x02) == 0x02 || (BufStatus & 0x08) == 0x08)
                    {
                        Console.WriteLine("Error code 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                        HSDAQNet.DATALOG.HS_StopLogger(hHS);
                        break;
                    }
                    Console.Write(BufferCnt + "\r");
                }
                if (Console.KeyAvailable)
                {
                    if (Console.ReadKey(true).Key == ConsoleKey.Q)
                    {
                        Thread.Sleep(10);
                        HSDAQNet.DATALOG.HS_StopLogger(hHS);
                        quit = true;
                    }
                }
            }
            HSDAQNet.Sys.HS_Device_Release(hHS);
            Console.WriteLine("\nThe file is stored in " + path);

            /******************************How to get log file****************************/
            int fileNum = 0;
            string STARTtime = "";
            string STARTdate = "";
            uint logcount = 0;
            string logpath = "";
            fileNum = HSDAQNet.DATALOG.HS_GetAllLogFiles(path, filetype);
            Console.WriteLine("fileNum : {0}", fileNum);
            for (int i = 0; i < fileNum; i++)
            {
                IntPtr loghandle = HSDAQNet.DATALOG.HS_LogFile_Open_byIndex(i, ref logpath);
                HSDAQNet.DATALOG.HS_GetLogFile_AIScanSampleInfo(loghandle, ref logcount, ref STARTdate, ref STARTtime);
                Console.WriteLine("Open file : " + logpath);
                float[] fdatabuffer = new float[logcount]; Console.WriteLine(logcount);
                HSDAQNet.DATALOG.HS_GetLogFile_AIData(loghandle, 0, logcount, fdatabuffer);
                for (int y = 0; y < 40; y++)
                {
                    if (y % ch == ch - 1)
                        Console.Write(fdatabuffer[y].ToString("F4") + "\r\n");
                    else
                        Console.Write(fdatabuffer[y].ToString("F4") + ",");

                }
                HSDAQNet.DATALOG.HS_LogFile_Close(loghandle);
            }
            /*****************************************************************************/
            Console.WriteLine("\nPress any key to close...");
            Console.ReadKey();
        }
    }
}
