﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.IO;
using System.Text.RegularExpressions;

namespace PETAR400_Analog_Threshold_Trigger
{
    class Program
    {
        public static bool IsIP(string ip)  //Check the IP format
        {
            return Regex.IsMatch(ip, @"^((2[0-4]\d|25[0-5]|[01]?\d\d?)\.){3}(2[0-4]\d|25[0-5]|[01]?\d\d?)$");
        }

        static void Main(string[] args)
        {
            bool quit = false, ret = false;
            string IP = "";
            IntPtr hHS = IntPtr.Zero;
            ushort BufStatus = 0;
            uint BufferCnt = 0;

            Console.WriteLine("!!This program demonstrates how to use Analog Input Trigger!!");
            Console.WriteLine("PET-AR400 has 4 AI channel");
            while (!quit)
            {
                Console.WriteLine("Please Enter IP!!");
                IP = Console.ReadLine();
                ret = IsIP(IP);
                if (ret == false)
                {
                    Console.WriteLine("Please enter the correct IP");
                }
                else
                {
                    quit = true;
                    Thread.Sleep(100);
                }
            }
            quit = false;
            hHS = HSDAQNet.Sys.HS_Device_Create(IP);
            if (hHS == null || hHS == IntPtr.Zero)
            {
                Console.WriteLine("Connect Fail");
                Console.WriteLine("\nPress any key to close...");
                Console.ReadKey();
                return;
            }
            string ModuleName = HSDAQNet.Sys.HS_GetModelName(hHS);
            Console.WriteLine("Module Name      : " + ModuleName);
            Console.WriteLine("Firmware Version : " + HSDAQNet.Sys.GetFirmwareVersion(hHS));
            Console.WriteLine("HSDAQ Version    : V." + HSDAQNet.Sys.GetSDKVersion());
            Console.WriteLine("HSDAQNet Version : V." + HSDAQNet.Sys.GetHSDAQNetVersion());
            /* For details, please refer to the API manual
             * AI CH input Range : 1 ~ 4
             * Gain Range        : 0
             * Mode Range        : 0 、 6
             * Sample rate       : 
             *  ch=1    : 12.8k/16k/32k/64k/128k Hz
             *  ch=2    : 12.8k/16k/32k/64k Hz
             *  ch=3/4  : 12.8k/16k/32k Hz
             * targetcnt         : 0 (continue mode)
             * AI trigger mode   : 0 ~ 3
             *    0:  Above
                  1:  Below
                  2:  Out-Range
                  3:  In-Range
            */
            short ch = 4, gain = 0, mode = 6, DTM = 0, Autorun = 0;
            int samplerate = 16000, AImode = 2;
            uint targetcnt = 4000, left_cnt = 0, right_cnt = 4000;
            byte[] AI_EN = new byte[4] { (byte)1, (byte)1, (byte)1, (byte)1 };// 1: enable , 0: disable
            float[] HI_level = new float[4] { 2, 2, 2, 2 };
            float[] Low_level = new float[4] { (float)1.5, (float)1.5, (float)1.5, (float)1.5 };
            float[] fdatabuf = new float[targetcnt];
            string path = Directory.GetCurrentDirectory();
            HSDAQNet.HSIO.HS_SetAIScanParam(hHS, ch, gain, mode, samplerate, targetcnt, DTM, Autorun);
            Thread.Sleep(10);
            HSDAQNet.HSIO.HS_GetAIScanParam(hHS, ref ch, ref gain, ref mode, ref samplerate, ref targetcnt, ref DTM, ref Autorun);
            //Set the analog trigger range, as long as any channel condition is met, the data will be returned 
            ret = HSDAQNet.HSIO.HS_SetAIAnalogTriggerParam(hHS, AImode, AI_EN, HI_level, Low_level, ch, left_cnt, right_cnt, 0);
            Console.WriteLine("Channel    : " + ch);
            Console.WriteLine("Gain       : " + gain);
            Console.WriteLine("Mode       : " + mode);
            Console.WriteLine("Samplerate : " + samplerate);
            Console.WriteLine("targetcnt  : " + targetcnt);
            Console.WriteLine("---------------------------------------------------------");
            Console.WriteLine("Start!");
            HSDAQNet.HSIO.HS_StartAIScan(hHS);
            while (!quit)
            {
                bool result = HSDAQNet.HSIO.HS_GetAIBufferStatus(hHS, ref BufStatus, ref BufferCnt);
                if (result)
                {
                    if ((BufStatus & 0x02) == 0x02 || (BufStatus & 0x08) == 0x08)
                    {
                        Console.WriteLine("Error code 0x" + HSDAQNet.ErrHandling.GetLastError().ToString("x8"));
                        HSDAQNet.HSIO.HS_StopAIScan(hHS);
                        break;
                    }
                    Console.Write(BufferCnt + "\r");
                }
                if (BufferCnt >= targetcnt)
                {
                    HSDAQNet.HSIO.HS_GetAIBuffer(hHS, fdatabuf, targetcnt);
                    HSDAQNet.HSIO.HS_StopAIScan(hHS);
                    quit = true;
                }
            }
            for (int i = 0; i < 400; i++)
            {
                if (i % ch == ch - 1)
                    Console.Write(fdatabuf[i].ToString("F4") + "\r\n");
                else
                    Console.Write(fdatabuf[i].ToString("F4") + ",");
            }
            HSDAQNet.Sys.HS_Device_Release(hHS);
            Console.WriteLine("\nTotal reads : " + BufferCnt);
            Console.WriteLine("\nPress any key to close...");
            Console.ReadKey();
        }
    }
}
